@@ -26,6 +26,12 @@
 #include "picture_viewer.h"
 #include "main_window.h"
 
+typedef struct {
+    RsttoNavigator *navigator;
+    RsttoMainWindow *main_window;
+    GSList *files;
+} RsttoOpenFiles;
+
 static ThunarVfsMimeDatabase *mime_dbase = NULL;
 
 static XfceRc *xfce_rc;
@@ -38,6 +44,9 @@ rstto_window_save_geometry_timer_destroy(gpointer user_data);
 static gboolean
 cb_rstto_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event);
 
+static gboolean
+cb_rstto_open_files (RsttoOpenFiles *rof);
+
 gboolean version = FALSE;
 
 static GOptionEntry entries[] =
@@ -49,6 +58,7 @@ static GOptionEntry entries[] =
     { NULL }
 };
 
+
 #if GTK_CHECK_VERSION(2,12,0)
 #define RSTTO_COLOR_PARSE gdk_color_parse
 #define RSTTO_COLOR_TO_STRING gdk_color_to_string
@@ -238,6 +248,7 @@ int main(int argc, char **argv)
     gint max_cache = xfce_rc_read_int_entry(xfce_rc, "MaxImagesCacheSize", 64);
     gboolean preload_during_slideshow = xfce_rc_read_bool_entry (xfce_rc, "PreloadDuringSlideShow", FALSE);
     gboolean override_bg_color = xfce_rc_read_bool_entry (xfce_rc, "OverrideBgColor", FALSE);
+    gboolean scale_to_100 = xfce_rc_read_bool_entry (xfce_rc, "ScaleTo100", FALSE);
 
     if (override_bg_color)
     {
@@ -258,6 +269,7 @@ int main(int argc, char **argv)
 
     rstto_main_window_set_max_cache_size(RSTTO_MAIN_WINDOW(window), max_cache);
     rstto_main_window_set_slideshow_timeout(RSTTO_MAIN_WINDOW(window), (gdouble)slideshow_timeout);
+    rstto_main_window_set_scale_to_100(RSTTO_MAIN_WINDOW(window), scale_to_100);
 
     GtkRecentManager *recent_manager = rstto_main_window_get_recent_manager(RSTTO_MAIN_WINDOW(window));
     rstto_navigator_set_timeout(navigator, slideshow_timeout);
@@ -265,188 +277,19 @@ int main(int argc, char **argv)
     /* When more then one file is provided over the CLI,
      * just open those files and don't index the folder
      */
-    if (argc > 2)
+    if (argc > 1)
     {
+        RsttoOpenFiles rof;
+
+        rof.files = NULL;
+        rof.navigator = navigator;
+        rof.main_window = RSTTO_MAIN_WINDOW(window);
         for (n = 1; n < argc; ++n)
         {
-            ThunarVfsPath *path;
-            if (g_path_is_absolute(argv[n]))
-                path = thunar_vfs_path_new(argv[n], NULL);
-            else
-            {
-                gchar *base_dir = g_get_current_dir();
-
-                path_dir = g_build_path("/", base_dir, argv[n], NULL);
-                path = thunar_vfs_path_new(path_dir, NULL);
-
-                g_free(base_dir);
-            }
-
-            if (path)
-            {
-
-                ThunarVfsInfo *info = thunar_vfs_info_new_for_path(path, NULL);
-                if(info)
-                {
-                    if(strcmp(thunar_vfs_mime_info_get_name(info->mime_info), "inode/directory"))
-                    {
-                        gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
-                        if(!strcmp(file_media, "image"))
-                        {
-                            RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, info);
-                            rstto_navigator_add (navigator, entry, TRUE);
-                        }
-                    }
-                    else
-                    {
-                        GDir *dir = g_dir_open(argv[n], 0, NULL);
-                        const gchar *filename = g_dir_read_name(dir);
-                        rstto_navigator_set_monitor_handle_for_dir(navigator, info->path);
-
-                        while (filename)
-                        {
-                            gchar *path_name = g_strconcat(argv[n],  "/", filename, NULL);
-                            ThunarVfsPath *file_path = thunar_vfs_path_new(path_name, NULL);
-                            if (file_path)
-                            {
-                                ThunarVfsInfo *file_info = thunar_vfs_info_new_for_path(file_path, NULL);
-                                gchar *file_media = thunar_vfs_mime_info_get_media(file_info->mime_info);
-                                if(!strcmp(file_media, "image"))
-                                {
-                                    RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, file_info);
-                                    rstto_navigator_add (navigator, entry, FALSE);
-                                }
-                                g_free(file_media);
-                                thunar_vfs_path_unref(file_path);
-                            }
-                            g_free(path_name);
-                            filename = g_dir_read_name(dir);
-                        }
-                        g_dir_close(dir);
-                    }
-
-                    gchar *uri = thunar_vfs_path_dup_uri(info->path);
-                    gtk_recent_manager_add_item(recent_manager, uri);
-                    g_free(uri);
-                }
-            }
+            rof.files = g_slist_prepend(rof.files, argv[n]);
         }
-        rstto_navigator_jump_first(navigator);
-    }
-    else
-    {
-        if (argc == 2)
-        {
-            ThunarVfsPath *path;
-            if (g_path_is_absolute(argv[1]))
-            {
-                path = thunar_vfs_path_new(argv[1], NULL);
-                path_dir = g_strdup(argv[1]);
-            }
-            else
-            {
-                gchar *base_dir = g_get_current_dir();
 
-                path_dir = g_build_path("/", base_dir, argv[1], NULL);
-                path = thunar_vfs_path_new(path_dir, NULL);
-
-                g_free(base_dir);
-            }
-
-            if (path)
-            {
-
-                ThunarVfsInfo *info = thunar_vfs_info_new_for_path(path, NULL);
-                if(info)
-                {
-                    if(strcmp(thunar_vfs_mime_info_get_name(info->mime_info), "inode/directory"))
-                    {
-                        gchar* media = thunar_vfs_mime_info_get_media(info->mime_info);
-                        if(!strcmp(media, "image"))
-                        {
-                            ThunarVfsPath *_path = thunar_vfs_path_get_parent(path);
-                            thunar_vfs_path_unref(path);
-                            path = _path;
-
-                            gchar *path_string = thunar_vfs_path_dup_string(path);
-                            
-                            GDir *dir = g_dir_open(path_string, 0, NULL);
-                            const gchar *filename = g_dir_read_name(dir);
-                            while (filename)
-                            {
-                                gchar *path_name = g_strconcat(path_string,  "/", filename, NULL);
-                                ThunarVfsPath *file_path = thunar_vfs_path_new(path_name, NULL);
-                                if (file_path)
-                                {
-                                    ThunarVfsInfo *file_info = thunar_vfs_info_new_for_path(file_path, NULL);
-                                    gchar *file_media = thunar_vfs_mime_info_get_media(file_info->mime_info);
-                                    if(!strcmp(file_media, "image"))
-                                    {
-                                        RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, file_info);
-                                        gint i = rstto_navigator_add (navigator, entry, FALSE);
-                                        if (path_dir == NULL)
-                                        {
-                                            if (!strcmp(path_name, argv[1]))
-                                            {
-                                                rstto_navigator_set_file_nr(navigator, i);
-                                            }
-                                        }
-                                        else
-                                        {
-                                            if (!strcmp(path_name, path_dir))
-                                            {
-                                                rstto_navigator_set_file_nr(navigator, i);
-                                            }
-
-                                        }
-                                    }
-                                    g_free(file_media);
-                                    thunar_vfs_path_unref(file_path);
-                                }
-                                g_free(path_name);
-                                filename = g_dir_read_name(dir);
-                            }
-                            g_dir_close(dir);
-                            g_free(path_string);
-                        }
-                        g_free(media);
-                    }
-                    else
-                    {
-                        GDir *dir = g_dir_open(argv[1], 0, NULL);
-                        const gchar *filename = g_dir_read_name(dir);
-
-                        rstto_navigator_set_monitor_handle_for_dir(navigator, info->path);
-                        while (filename)
-                        {
-                            gchar *path_name = g_strconcat(path_dir,  "/", filename, NULL);
-                            ThunarVfsPath *file_path = thunar_vfs_path_new(path_name, NULL);
-                            if (file_path)
-                            {
-                                ThunarVfsInfo *file_info = thunar_vfs_info_new_for_path(file_path, NULL);
-                                gchar *file_media = thunar_vfs_mime_info_get_media(file_info->mime_info);
-                                if(!strcmp(file_media, "image"))
-                                {
-                                    RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, file_info);
-                                    rstto_navigator_add (navigator, entry, FALSE);
-                                }
-                                g_free(file_media);
-                                thunar_vfs_path_unref(file_path);
-                            }
-                            g_free(path_name);
-                            filename = g_dir_read_name(dir);
-                        }
-                        rstto_navigator_jump_first(navigator);
-                        g_dir_close(dir);
-                    }
-                    gchar *uri = thunar_vfs_path_dup_uri(info->path);
-                    gtk_recent_manager_add_item(recent_manager, uri);
-                    g_free(uri);
-                }
-                thunar_vfs_path_unref(path);
-            }
-
-        }
+        gtk_init_add((GtkFunction)cb_rstto_open_files, &rof);
     }
 
     g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
@@ -479,6 +322,7 @@ int main(int argc, char **argv)
     bg_color = (GdkColor *)rstto_main_window_get_pv_bg_color(RSTTO_MAIN_WINDOW(window));
 
     xfce_rc_write_bool_entry(xfce_rc, "ShowToolBar", rstto_main_window_get_show_toolbar(RSTTO_MAIN_WINDOW(window)));
+    xfce_rc_write_bool_entry(xfce_rc, "ScaleTo100", rstto_main_window_get_scale_to_100(RSTTO_MAIN_WINDOW(window)));
     xfce_rc_write_bool_entry(xfce_rc, "PreloadDuringSlideShow", navigator->preload);
     xfce_rc_write_bool_entry(xfce_rc, "ShowThumbnailViewer", rstto_main_window_get_show_thumbnail_viewer(RSTTO_MAIN_WINDOW(window)));
     if (bg_color)
@@ -562,3 +406,62 @@ cb_rstto_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *even
     /* let Gtk+ handle the configure event */
     return FALSE;
 }
+
+static gboolean
+cb_rstto_open_files (RsttoOpenFiles *rof)
+{
+    gchar *path_dir = NULL;
+    RsttoNavigator *navigator = rof->navigator;
+    RsttoMainWindow *window = rof->main_window;
+
+    GtkStatusbar *bar = rstto_main_window_get_statusbar(window);
+
+    guint context_id = gtk_statusbar_get_context_id(bar, "StatusMessages");
+    guint message_id = gtk_statusbar_push(bar, context_id, N_("Opening file(s)..."));
+
+    if (g_slist_length(rof->files) >= 1)
+    {
+        GSList *_iter = rof->files;
+        while(_iter)
+        {
+            if (g_path_is_absolute(_iter->data))
+            {
+                path_dir = g_strdup(_iter->data);
+            }
+            else
+            {
+                gchar *base_dir = g_get_current_dir();
+
+                path_dir = g_build_path("/", base_dir, _iter->data, NULL);
+
+                g_free(base_dir);
+            }
+            if(g_file_test(path_dir, G_FILE_TEST_EXISTS))
+            {
+
+                if(g_file_test(path_dir, G_FILE_TEST_IS_DIR))
+                {
+                    if(rstto_navigator_open_folder (navigator, path_dir, TRUE, NULL) == TRUE)
+                    {
+                        rstto_navigator_jump_first(navigator);
+                        gtk_statusbar_remove(bar, context_id, message_id);
+                    }
+                }
+                else
+                {
+                    rstto_navigator_open_file (navigator, path_dir, TRUE, NULL);
+                    gtk_statusbar_remove(bar, context_id, message_id);
+                }
+            }
+
+            g_free(path_dir);
+
+            _iter = g_slist_next(_iter);
+        }
+
+        if (g_slist_length(rof->files) > 1)
+            rstto_navigator_jump_first(navigator);
+    }
+
+    return FALSE;
+}
