@@ -59,6 +59,7 @@ struct _RsttoMainWindowPriv
         gint            max_cache_size;
         gdouble         slideshow_timeout;
         const GdkColor *bg_color;
+        gboolean        scale_to_100;
     } settings;
 
     struct {
@@ -292,6 +293,8 @@ rstto_main_window_init(RsttoMainWindow *window)
 
     window->priv->mime_dbase = thunar_vfs_mime_database_get_default();
     window->priv->icon_theme = gtk_icon_theme_get_default();
+
+    window->priv->settings.scale_to_100 = TRUE;
     
 
     GtkAccelGroup *accel_group = gtk_accel_group_new();
@@ -299,11 +302,11 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_window_set_title(GTK_WINDOW(window), PACKAGE_STRING);
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
-    window->priv->navigator = rstto_navigator_new();
+    window->priv->manager = gtk_recent_manager_get_default();
+    window->priv->navigator = rstto_navigator_new(window->priv->manager);
     window->priv->thumbnail_viewer = rstto_thumbnail_bar_new(window->priv->navigator);
     window->priv->picture_viewer = rstto_picture_viewer_new(window->priv->navigator);
 
-    window->priv->manager = gtk_recent_manager_get_default();
 
 /* Set up default settings */
     window->priv->settings.thumbnail_viewer_orientation = GTK_ORIENTATION_HORIZONTAL; 
@@ -979,6 +982,18 @@ rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_si
     rstto_navigator_set_max_history_size(window->priv->navigator, max_cache_size * 1000000);
 }
 
+void
+rstto_main_window_set_scale_to_100 (RsttoMainWindow *window, gboolean scale_to_100)
+{
+    window->priv->settings.scale_to_100 = scale_to_100;
+}
+
+gboolean
+rstto_main_window_get_scale_to_100 (RsttoMainWindow *window)
+{
+    return window->priv->settings.scale_to_100;
+}
+
 /* CALLBACK FUNCTIONS */
 
 static void
@@ -1127,6 +1142,17 @@ cb_rstto_main_window_state_event(GtkWidget *widget, GdkEventWindowState *event,
             }
         }
     }
+    if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED)
+    {
+        RsttoNavigatorEntry *entry = rstto_navigator_get_file(window->priv->navigator);
+
+        if (window->priv->settings.scale_to_100 == TRUE)
+        {
+            rstto_picture_viewer_set_zoom_mode(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), RSTTO_ZOOM_MODE_CUSTOM);
+            rstto_navigator_entry_set_scale(entry, 0);
+        }
+
+    }
 }
 
 static void
@@ -1161,6 +1187,12 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     GtkWidget *slideshow_main_lbl;
     GtkWidget *display_main_vbox;
     GtkWidget *display_main_lbl;
+    GtkWidget *behaviour_main_vbox;
+    GtkWidget *behaviour_main_lbl;
+
+    GtkWidget *resize_to_content_vbox, *resize_to_content_frame;
+    GtkWidget *resize_on_maximize_check;
+
     GtkWidget *bg_color_vbox;
     GtkWidget *bg_color_hbox;
     GtkWidget *bg_color_frame;
@@ -1192,6 +1224,11 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook,  TRUE, TRUE, 0);
 
+/** Add notebook pages */
+    behaviour_main_vbox = gtk_vbox_new(FALSE, 0);
+    behaviour_main_lbl = gtk_label_new(_("Behaviour"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), behaviour_main_vbox, behaviour_main_lbl);
+
     slideshow_main_vbox = gtk_vbox_new(FALSE, 0);
     slideshow_main_lbl = gtk_label_new(_("Slideshow"));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), slideshow_main_vbox, slideshow_main_lbl);
@@ -1200,6 +1237,19 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     display_main_lbl = gtk_label_new(_("Display"));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), display_main_vbox, display_main_lbl);
 
+/** Add content for behaviour page */
+    resize_to_content_vbox = gtk_vbox_new(FALSE, 0);
+    resize_to_content_frame = xfce_create_framebox_with_content(_("Scaling"), resize_to_content_vbox);
+
+    resize_on_maximize_check = gtk_check_button_new_with_mnemonic(_("Don't scale over 100% when maximizing the window."));
+    gtk_box_pack_start(GTK_BOX(resize_to_content_vbox), resize_on_maximize_check, FALSE, TRUE, 0);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resize_on_maximize_check), window->priv->settings.scale_to_100);
+
+    gtk_container_set_border_width (GTK_CONTAINER (resize_to_content_frame), 8);
+    gtk_box_pack_start(GTK_BOX(behaviour_main_vbox), resize_to_content_frame, FALSE, TRUE, 0);
+
+/** Add content for display page */
     bg_color_vbox = gtk_vbox_new(FALSE, 0);
     bg_color_frame = xfce_create_framebox_with_content (_("Background Color"), bg_color_vbox);
 
@@ -1244,6 +1294,7 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     gtk_box_pack_start(GTK_BOX(display_main_vbox), bg_color_frame, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(display_main_vbox), cache_frame, FALSE, TRUE, 0);
 
+/** Add content for slideshow page */
     GtkWidget *slideshow_vbox = gtk_vbox_new(FALSE, 0);
     GtkWidget *slideshow_frame = xfce_create_framebox_with_content (_("Timeout"), slideshow_vbox);
 
@@ -1301,6 +1352,8 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
             }
             rstto_picture_viewer_redraw(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
             rstto_main_window_set_max_cache_size(window, GTK_ADJUSTMENT(cache_adjustment)->value);
+
+            window->priv->settings.scale_to_100 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resize_on_maximize_check));
         default:
             break;
     }
@@ -1350,6 +1403,12 @@ cb_rstto_main_window_quit(GtkWidget *widget, RsttoMainWindow *window)
 static void
 cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
 {
+    GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+
+    gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
+    gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
+
     GtkWidget *dialog = gtk_file_chooser_dialog_new(_("Open image"),
                                                     GTK_WINDOW(window),
                                                     GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -1361,24 +1420,7 @@ cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
     if(response == GTK_RESPONSE_OK)
     {
         const gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-
-        ThunarVfsPath *path = thunar_vfs_path_new(filename, NULL);
-        if (path)
-        {
-            ThunarVfsInfo *info = thunar_vfs_info_new_for_path(path, NULL);
-            gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
-            if(!strcmp(file_media, "image"))
-            {
-                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                rstto_navigator_add (window->priv->navigator, entry, TRUE);
-                gchar *uri = thunar_vfs_path_dup_uri(info->path);
-                gtk_recent_manager_add_item(window->priv->manager, uri);
-                g_free(uri);
-            }
-            g_free(file_media);
-            thunar_vfs_path_unref(path);
-        }
-        else
+        if (!rstto_navigator_open_file(window->priv->navigator, filename, FALSE, NULL))
         {
             gtk_widget_destroy(dialog);
             dialog = gtk_message_dialog_new(GTK_WINDOW(window),
@@ -1390,11 +1432,23 @@ cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
         }
     }
     gtk_widget_destroy(dialog);
+
+    if (statusbar)
+    {
+        gtk_statusbar_remove(statusbar, context_id, message_id);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    }
 }
 
 static void
 cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
 {
+    GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+
+    gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
+    gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
+
     GtkWidget *dialog = gtk_file_chooser_dialog_new(_("Open folder"),
                                                     GTK_WINDOW(window),
                                                     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
@@ -1405,101 +1459,61 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
     gint response = gtk_dialog_run(GTK_DIALOG(dialog));
     if(response == GTK_RESPONSE_OK)
     {
-        rstto_navigator_clear(window->priv->navigator);
+        gtk_widget_hide(dialog);
         const gchar *dir_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-        GDir *dir = g_dir_open(dir_name, 0, NULL);
-        if (dir)
-        {
-            ThunarVfsPath *dir_path = thunar_vfs_path_new(dir_name, NULL);
-            if (dir_path)
-            {
-                rstto_navigator_set_monitor_handle_for_dir(window->priv->navigator, dir_path);
-                thunar_vfs_path_unref(dir_path);
-                dir_path = NULL;
-            }
 
-            const gchar *filename = g_dir_read_name(dir);
-            while (filename)
-            {
-                gchar *path_name = g_strconcat(dir_name,  "/", filename, NULL);
-                ThunarVfsPath *path = thunar_vfs_path_new(path_name, NULL);
-                if (path)
-                {
-                    ThunarVfsInfo *info = thunar_vfs_info_new_for_path(path, NULL);
-                    gchar *file_media = thunar_vfs_mime_info_get_media(info->mime_info);
-                    if(!strcmp(file_media, "image"))
-                    {
-                        RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                        rstto_navigator_add (window->priv->navigator, entry, FALSE);
-                    }
-                    g_free(file_media);
-                    thunar_vfs_path_unref(path);
-                }
-                g_free(path_name);
-                filename = g_dir_read_name(dir);
-            }
-            rstto_navigator_jump_first(window->priv->navigator);
-            gchar *uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
-            gtk_recent_manager_add_item(window->priv->manager, uri);
-            g_free(uri);
-            g_dir_close(dir);
+        RsttoNavigator *navigator = window->priv->navigator;
+        if(rstto_navigator_open_folder(navigator, dir_name, TRUE, NULL) == TRUE)
+        {
+            rstto_navigator_jump_first(navigator);
         }
     }
     gtk_widget_destroy(dialog);
+
+    if (statusbar)
+    {
+        gtk_statusbar_remove(statusbar, context_id, message_id);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    }
 }
 
 static void
 cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *window)
 {
+    GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+
+    gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
+    gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
+
     gchar *uri = gtk_recent_chooser_get_current_uri(chooser);
-    ThunarVfsPath *path = thunar_vfs_path_new(uri, NULL);
-    if (path)
+    ThunarVfsPath *vfs_path = thunar_vfs_path_new(uri, NULL);
+    if (vfs_path)
     {
-        ThunarVfsInfo *info = thunar_vfs_info_new_for_path(path, NULL);
-        if(info)
+        gchar *path = thunar_vfs_path_dup_string(vfs_path);
+        if(g_file_test(path, G_FILE_TEST_EXISTS))
         {
-            if(strcmp(thunar_vfs_mime_info_get_name(info->mime_info), "inode/directory"))
+            if(g_file_test(path, G_FILE_TEST_IS_DIR))
             {
-                RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, info);
-                rstto_navigator_add (window->priv->navigator, entry, TRUE);
+                RsttoNavigator *navigator = window->priv->navigator;
+                if(rstto_navigator_open_folder(navigator, path, TRUE, NULL) == TRUE)
+                {
+                    rstto_navigator_jump_first(navigator);
+                }
             }
             else
             {
-                rstto_navigator_clear(window->priv->navigator);
-                gchar *dir_path = thunar_vfs_path_dup_string(info->path);
-                GDir *dir = g_dir_open(dir_path, 0, NULL);
-                rstto_navigator_set_monitor_handle_for_dir(window->priv->navigator, info->path);
-
-                const gchar *filename = g_dir_read_name(dir);
-                while (filename)
-                {
-                    gchar *path_name = g_strconcat(dir_path,  "/", filename, NULL);
-                    ThunarVfsPath *file_path = thunar_vfs_path_new(path_name, NULL);
-                    if (file_path)
-                    {
-                        ThunarVfsInfo *file_info = thunar_vfs_info_new_for_path(file_path, NULL);
-                        gchar *file_media = thunar_vfs_mime_info_get_media(file_info->mime_info);
-                        if(!strcmp(file_media, "image"))
-                        {
-                            RsttoNavigatorEntry *entry = rstto_navigator_entry_new(window->priv->navigator, file_info);
-                            rstto_navigator_add (window->priv->navigator, entry, FALSE);
-                        }
-                        g_free(file_media);
-                        thunar_vfs_path_unref(file_path);
-                    }
-                    g_free(path_name);
-                    filename = g_dir_read_name(dir);
-                }
-                rstto_navigator_jump_first(window->priv->navigator);
-                g_free(dir_path);
+                rstto_navigator_open_file(window->priv->navigator, path, FALSE, NULL);
             }
-            gchar *uri = thunar_vfs_path_dup_uri(info->path);
-            gtk_recent_manager_add_item(window->priv->manager, uri);
-            g_free(uri);
         }
-        thunar_vfs_path_unref(path);
+        thunar_vfs_path_unref(vfs_path);
     }
 
+    if (statusbar)
+    {
+        gtk_statusbar_remove(statusbar, context_id, message_id);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    }
 }
 
 static void
@@ -1911,6 +1925,11 @@ cb_rstto_bg_color_override_check_toggled(GtkToggleButton *button, GtkWidget *wid
     else
     {
         gtk_widget_set_sensitive(widget, FALSE);
-
     }
 }
+
+GtkStatusbar *
+rstto_main_window_get_statusbar(RsttoMainWindow *window)
+{
+    return GTK_STATUSBAR(window->priv->statusbar);
+}
